@@ -61,17 +61,6 @@ DISPLAYS         = {}
 TIMERS           = {}
 TASK_CHECKING    = {}
 
-#def client_is_active(client):
-#    try:
-#        transport  = client.get_transport()
-#        if not transport: return False
-#        transport.send_ignore()106  def md5_of_content(cont):
-#        pass
-#    except Exception as e:
-#        return False
-#        pass
-#    return True
-
 def to_int_number(s):
     try:
         return int(s)
@@ -136,7 +125,7 @@ def get_exec_connection(config):
             ssh_client.connect( hostname   =  config['server_address'],
                             port       =  config['server_port'], 
                             username   =  config['auth_username'], 
-                            password   =  config['auth_password'] )
+                            password   =  base64.b64decode( config['auth_password'] ) )
             pass
 
         else:
@@ -190,19 +179,6 @@ async def test_ssh_connection(websocket, config):
     await websocket.send(res)
     pass
 
-#async def select_sftp_sync_folder(websocket):
-#    print("try select sftp sync folder")
-#    import tkinter as tk
-#    from tkinter.filedialog import askdirectory
-#    root = tk.Tk()
-#    dirname = askdirectory()
-#    print( 'filder = ', dirname)
-#    await websocket.send( json.dumps({
-#        "name"   : "select-sftp-sync-folder",
-#        "path"   : dirname,
-#    }))
-#
-#    pass
 async def select_sftp_sync_folder(websocket):
     try:
         desktop_pidl = shell.SHGetFolderLocation(0, shellcon.CSIDL_DESKTOP, 0, 0)
@@ -289,28 +265,94 @@ fi
         #log("preperation done without error")
 
         # check pbs system info
-        (pbs_exist, queue_info) = check_if_pbs_exist(client)
+        (slurm_exist, queue_info) = check_if_slurm_exist(client)
+        if not slurm_exist: (pbs_exist,   queue_info) = check_if_pbs_exist(client)
         if check_queue:
+            queue_type   = "slurm" if slurm_exist else ("pbs" if pbs_exist else "none")
+            print("return queue check info", queue_type, queue_info)
             await websocket.send( json.dumps({
-                "name"   : "submit-task",
-                "ckey"   : ckey,
-                "uid"    : task_uid,
-                "info"   : "queue_required" ,
-                "queue"  : queue_info,
+                "name"         : "submit-task",
+                "ckey"         : ckey,
+                "uid"          : task_uid,
+                "info"         : "queue_required" ,
+                "queue"        : queue_info,
+                "queue_type"   : queue_type,
             }))
             return
 
         #print('task_options=',task_options)
-        gpu_resource = (":gpus="+str(task_options.get("gpus")) ) if task_options.get("gpus",None) else ""
-        #print('gpu=>>>', gpu_resource, "<<<", sep='' )
         #print('deal tasks:', batch_codes)
         for task_batch in batch_codes:
             task_code = batch_codes[task_batch]
             #print("deal task", task_batch, task_code )
-            if task_queue and pbs_exist:
+            if task_queue and slurm_exist:
+                running_mode   = 'slurm'
                 #print("run task with pbs system")
-                #print("try submit task to pbs", task_options)
+                gpu_resource = ("#SBATCH --gres=gpu:"+str(task_options.get("gpus")) ) if task_options.get("gpus",None) else ""
+                #print("try submit task to slurm", task_options, gpu_resource)
+                task_queue = task_options.get("queue")
+                if task_queue[-1] == "*": task_queue = task_queue[:-1]
+                stdin, stdout, stderr = client.exec_command( """ 
+. /etc/profile
+. ~/.bashrc
+cd ~/pond-calculation-works/{2}/
+cat << 'EOF' > .tasks/{0}.code
+{1}
+EOF
+cat <<EOF > .tasks/{0}.sh
+#!/bin/bash
+#SBATCH --job-name={0}
+#SBATCH --nodes={3}
+#SBATCH --ntasks-per-node={4}
+{5}
+#SBATCH -p {6}
+#SBATCH -o $HOME/pond-calculation-works/{2}/.tasks/{0}.log
+
+cd \$SLURM_SUBMIT_DIR
+pwd
+echo \`date +%s\` > ./.tasks/{0}.time
+if [ -f /etc/profile ]; then
+    . /etc/profile
+fi
+if [ -f \$HOME/.bashrc ]; then
+    . \$HOME/.bashrc
+fi
+if [ -f .bashrc ]; then 
+    . .bashrc
+fi
+
+./.tasks/{0}.code > ./.tasks/{0}.log 2>&1
+
+echo \`date +%s\` >> ./.tasks/{0}.time
+if [ -f ./.tasks/{0}.pid ]; then 
+    rm ./.tasks/{0}.pid;
+fi
+EOF
+chmod +x .tasks/{0}.code
+chmod +x .tasks/{0}.sh
+
+echo -n "" >./.tasks/{0}.log
+#sbatch ./.tasks/{0}.sh > ./.tasks/{0}.pid 2>&1
+sbatch ./.tasks/{0}.sh > ./.tasks/{0}.pid
+                """.format(
+                    task_uid+"_"+task_batch, task_code, ckey,
+                    task_options.get("nodes", 1),
+                    task_options.get("ppn", 1),
+                    gpu_resource,
+                    task_queue,
+                ) )
+                stdin.channel.shutdown_write()
+                stdout.channel.recv_exit_status()
+                out = (stdout.read() or b"").decode()
+                err = (stderr.read() or b"").decode()
+                #print("submit out:", out)
+                #print("submit err:", err)
+                pass
+            elif task_queue and pbs_exist:
                 running_mode   = 'pbs'
+                #print("run task with pbs system")
+                gpu_resource = (":gpus="+str(task_options.get("gpus")) ) if task_options.get("gpus",None) else ""
+                #print("try submit task to pbs", task_options, gpu_resource)
                 stdin, stdout, stderr = client.exec_command( """ 
 . /etc/profile
 . ~/.bashrc
@@ -348,7 +390,8 @@ chmod +x .tasks/{0}.code
 chmod +x .tasks/{0}.pbs
 
 echo -n "" >./.tasks/{0}.log
-qsub ./.tasks/{0}.pbs > ./.tasks/{0}.pid 2>&1
+#qsub ./.tasks/{0}.pbs > ./.tasks/{0}.pid 2>&1
+qsub ./.tasks/{0}.pbs > ./.tasks/{0}.pid
                 """.format(
                     task_uid+"_"+task_batch, task_code, ckey,
                     task_options.get("nodes", 1),
@@ -417,7 +460,6 @@ echo $! > ./.tasks/{0}.pid
                     "name"          : "submit-task",
                     "ckey"          : ckey,
                     "uid"           : task_uid,
-                    "pbs_exist"     : pbs_exist, 
                     "running_mode"  : running_mode,
                     "info"          : "task_launch_success",
                 }))
@@ -434,11 +476,28 @@ echo $! > ./.tasks/{0}.pid
 
     pass
 
+def check_if_slurm_exist(client):
+    stdin, stdout, stderr = client.exec_command("""
+. /etc/profile
+. ~/.bashrc
+echo "#-queue-out-#"
+sinfo
+    """)
+    stdin.channel.shutdown_write()
+    stdout.channel.recv_exit_status()
+    out = (stdout.read() or b"").decode()
+    err = (stderr.read() or b"").decode()
+    print("check if_slurm_exist out=", out)
+    print("check if_slurm_exist err=", err)
+    if err: return (False,'')
+    out = out.split("#-queue-out-#\n")[1]
+    return (True,out)
+
 def  check_if_pbs_exist(client):
     stdin, stdout, stderr = client.exec_command("""
 . /etc/profile
 . ~/.bashrc
-echo "#-qstat-out-#"
+echo "#-queue-out-#"
 qstat -Q
     """)
     stdin.channel.shutdown_write()
@@ -446,9 +505,9 @@ qstat -Q
     out = (stdout.read() or b"").decode()
     err = (stderr.read() or b"").decode()
     if err: return (False,'')
-    out = out.split("#-qstat-out-#\n")[1]
-    #print("check if _pbs_exist out=", out)
-    #print("check if _pbs_exist err=", err)
+    out = out.split("#-queue-out-#\n")[1]
+    #print("check if_pbs_exist out=", out)
+    #print("check if_pbs_exist err=", err)
     return (True,out)
 
 async def stop_task(config, ckey, message, websocket):
@@ -469,7 +528,40 @@ async def stop_task(config, ckey, message, websocket):
         return
     client      = conn['client']
     #print("try stop with :", running_mode)
-    if running_mode == 'pbs':
+    if running_mode == 'slurm':
+        print("check slurm task: {0}_{1}".format(task_uid, batch) )
+        if check_if_slurm_exist(client)[0]:
+            #print("check slurm task:", task_uid)
+            stdin, stdout, stderr = client.exec_command("""
+. /etc/profile
+. ~/.bashrc
+cd ~/pond-calculation-works/{2}/.tasks/
+if [ -f {0}_{1}.pid ]; then
+    scancel `cat {0}_{1}.pid |awk '{{print $NF}}'`
+    rm {0}_{1}.pid
+fi
+            """.format(task_uid, batch, ckey))
+            stdin.channel.shutdown_write()
+            stdout.channel.recv_exit_status()
+            out  = (stdout.read() or b"").decode()
+            err  = (stderr.read() or b"").decode()
+            print("task killed")
+            pass
+        else:
+            #print("pbs task kill, but no pbs command found" )
+            stdin, stdout, stderr = client.exec_command("""
+cd ~/pond-calculation-works/{2}/.tasks/
+if [ -f {0}_{1}.pid ]; then
+    rm {0}_{1}.pid
+fi
+            """.format(task_uid, batch, ckey))
+            stdin.channel.shutdown_write()
+            stdout.channel.recv_exit_status()
+            out  = (stdout.read() or b"").decode()
+            err  = (stderr.read() or b"").decode()
+            print("task already vanished")
+            pass
+    elif running_mode == 'pbs':
         print("check pbs task: {0}_{1}".format(task_uid, batch) )
         if check_if_pbs_exist(client)[0]:
             #print("check pbs task:", task_uid)
@@ -484,10 +576,8 @@ fi
             """.format(task_uid, batch, ckey))
             stdin.channel.shutdown_write()
             stdout.channel.recv_exit_status()
-            out = (stdout.read() or b"").decode()
-            err = (stderr.read() or b"").decode()
-            #print("kill out >>>", out, sep='')
-            #print("kill err >>>", err, sep='')
+            out  = (stdout.read() or b"").decode()
+            err  = (stderr.read() or b"").decode()
             print("task killed")
             pass
         else:
@@ -500,10 +590,8 @@ fi
             """.format(task_uid, batch, ckey))
             stdin.channel.shutdown_write()
             stdout.channel.recv_exit_status()
-            out = (stdout.read() or b"").decode()
-            err = (stderr.read() or b"").decode()
-            #print("kill out >>>", out, sep='')
-            #print("kill err >>>", err, sep='')
+            out  = (stdout.read() or b"").decode()
+            err  = (stderr.read() or b"").decode()
             print("task already vanished")
             pass
         pass
@@ -522,12 +610,12 @@ fi
         """.format(task_uid, batch, ckey))
         stdin.channel.shutdown_write()
         stdout.channel.recv_exit_status()
-        out = (stdout.read() or b"").decode()
-        err = (stderr.read() or b"").decode()
-        #print("out >>>",out,"<<<",sep='',flush=True)
-        #print("err >>>",err,"<<<",sep='',flush=True)
+        out  = (stdout.read() or b"").decode()
+        err  = (stderr.read() or b"").decode()
         print("task killed",flush=True)
         pass
+    print("kill out >>>", out, sep='')
+    print("kill err >>>", err, sep='')
     await websocket.send( json.dumps({
         "name"   : "check-task",
         "info"   : "task-stoped",
@@ -770,7 +858,7 @@ async def get_ssh_connection(websocket, config, ckey='', page_id = ''):
             client.connect( hostname   =  config['server_address'],
                             port       =  config['server_port'], 
                             username   =  config['auth_username'], 
-                            password   =  config['auth_password'] )
+                            password   =  base64.b64decode( config['auth_password'] ) )
             pass
         else:
             log("connect with no auth provided")
@@ -857,7 +945,7 @@ async def get_sftp_connection(websocket, config, ckey='', page_id = ''):
         elif config.get('auth_password',None):
             #print("connect with auth password", config['auth_password'])
             t.connect( username=config['auth_username'],
-                       password=config['auth_password'])
+                       password=base64.b64decode(config['auth_password']))
             pass
         else:
             log("connect with no auth provided")
